#!/usr/bin/perl
# The source of the tests for the paper
#   Sergej Znamenskij. "Local quadratic spline interpolating with cubic precision". 
#
# Prepared for the journal "Program Systems: Theory and Applications" at October 9, 2018
#
# (c) S. V. Znamenskij, 2018      CC-BY 4.0
# (c) Ailamazyan Program Systems Institute of RAS, 2018
use strict;
use warnings;
use Carp;
use Data::Dumper;
use Math::Trig;
use B::Deparse;

package Math::Spline::MingledSquare; 

sub new {
    my $type=shift;
    my $self=[];
    my @x=@{$_[0]};
    my @y=@{$_[1]};
    my @coeff2;
    my @m;
    my @delta;
    my $ai;
    my $a_i;
    croack("arrays x and y have different sizes $#x and $#y.\n") unless $#x==$#y;
    $m[0]=($y[1]-$y[0])/($x[1]-$x[0]);
    $m[1]=($y[2]-$y[1])/($x[2]-$x[1]);
    $delta[0]=($m[1]-$m[0])/($x[2]-$x[0]);
    foreach my $i (1 .. $#x-2){
        $m[$i+1]=($y[$i+2]-$y[$i+1])/($x[$i+2]-$x[$i+1]);
        $delta[$i]=($m[$i+1]-$m[$i])/($x[$i+2]-$x[$i]);
        next unless $i;
        $ai=sqrt(sqrt(sqrt(sqrt(sqrt(sqrt(sqrt(sqrt(abs($delta[$i])))))))));
        $a_i=sqrt(sqrt(sqrt(sqrt(sqrt(sqrt(sqrt(sqrt(abs($delta[$i-1])))))))));
        $coeff2[$i] = ($delta[$i-1]==$delta[$i])? $delta[$i]
            : ($delta[$i]+(((0.58*$a_i+0.42*$ai)/($ai+$a_i))*($x[$i+1]+$x[$i])-$x[$i+2])*($delta[$i]-$delta[$i-1])/($x[$i+2]-$x[$i-1])) ;

      }
      $coeff2[0] = ($delta[1]==$delta[0])? $delta[0]
        : ($delta[0]*($x[3]-$x[0]+$x[2]-0.5*($x[0]+$x[1])) +$delta[1]*( .5*($x[1]+$x[0]) -$x[2]))/($x[3]-$x[0]);
      $coeff2[$#x-1]=($delta[$#x-3]==$delta[$#x-2])? $delta[$#x-2]
        :  ($delta[$#x-2]*($x[$#x]-$x[$#x-3]+0.5*($x[$#x]+$x[$#x-1])-$x[$#x-2])-(0.5*($x[$#x]+$x[$#x-1])-$x[$#x-2])*($delta[$#x-3]))/($x[$#x]-$x[$#x-3]);
      push @{$self},[@x],[@y],[@m],[@coeff2],0;
      bless $self,$type;
}
sub evaluate {
  my ($self,$v)=@_;
  my $i=$self->[4];
  my $max_i;
  if ($v < $self->[0][$i] and $v> $self->[0][0]){
        ($i,$max_i)=(0,$i);}
    elsif($i > $#{$self->[0]}-2 or $v >$self->[0][$i+2]){
        $max_i=$#{$self->[0]};}
    else{
        if($v >$self->[0][$i+1]){
            $self->[4]= ++$i;}
        print $self->[1][$i]+($v-$self->[0][$i])*($self->[2][$i]+$self->[3][$i]*($v-$self->[0][$i+1])) ." = $self->[1][$i]+($v-$self->[0][$i])*($self->[2][$i]+$self->[3][$i]*($v-$self->[0][$i+1]\n" if $i == 2;
        return $self->[1][$i]+($v-$self->[0][$i])*($self->[2][$i]+$self->[3][$i]*($v-$self->[0][$i+1]));
        }
  my $k;
  while ($max_i-$i > 1) {
    $k=int(($max_i+$i)/2);
    if ($self->[0][$k]>$v) { $max_i=$k; } else { $i=$k; }}
  $self->[4]=$i;
  return $self->[1][$i]+($v-$self->[0][$i])*($self->[2][$i]+$self->[3][$i]*($v-$self->[0][$i+1]));
     }
        
package main;

my @task=({
    f=>sub{exp ($_[0])},
    x=>[-3..3],
    istep=>0.05,
    legendpos=>"-3,20",
    spy=>"\\coordinate (spypoint) at (axis cs:1.5,4.5 );\\coordinate (magnifyglass) at (axis cs:1,15);",
    fn=>'e^{x}'},
{# 1
    f=>sub{log (($_[0]-0.1))},
    x=>[1..6],
    istep=>0.05,
    legendpos=>"4.2,0.85",
    spy=>"\\coordinate (spypoint) at (axis cs:2.5,.88 );\\coordinate (magnifyglass) at (axis cs:1.7,1.4);",
    fn=>'\\ln(x-0.1)'},
{# 2
    f=>sub{1/ ($_[0])},
    x=>[1..6],
    istep=>0.05,
    legendpos=>"4.2,1.01",
    spy=>"\\coordinate (spypoint) at (axis cs:2.5,0.4);\\coordinate (magnifyglass) at (axis cs:2.7,0.8);",
    fn=>'1/x'},
{# 13
    f=>sub{atan($_[0]+.7)},
    x=>[-3..3],
    istep=>0.025,
    legendpos=>"-4.2,1.40",
    spy=>"\\coordinate (spypoint) at (axis cs:-0.5,.20 );\\coordinate (magnifyglass) at (axis cs:2.5,-.50);",
    fn=>'\\arctan(x+0.7)'},
{# 4
    f=>sub{exp (-0.2*($_[0]+.5)**2)},
    x=>[-8 .. 7],
    istep=>0.05,
    legendpos=>"-8.5,1.02",
    spy=>"\\coordinate (spypoint) at (axis cs:-1,.95);\\coordinate (magnifyglass) at (axis cs:4.8,.77);",
    # fn=>'\\exp (-0.1(x+0.5)^{2})'},
    fn=>'e^{-0.1(x+0.5)^{2}}'},
{# 5
    f=>sub{exp (-0.2*$_[0]**2)},
    x=>[-8 .. 7],
    istep=>0.05,
    legendpos=>"-8.5,1.02",
    spy=>"\\coordinate (spypoint) at (axis cs:0,0.96);\\coordinate (magnifyglass) at (axis cs:4.8,.77);",
    fn=>'e^{-0.1x^{2}}'},
#    fn=>'\\exp (-0.1x^{2})'},
{# 6
    f=>sub{exp (-0.05*$_[0] **2)},
    x=>[-8 .. 5],
    istep=>0.05,
    legendpos=>"-15.3,1",
    spy=>"\\coordinate (spypoint) at (axis cs:0,0.95 );\\coordinate (magnifyglass) at (axis cs:0,0.23);",
    fn=>'e^{-0.05x^{2}}'},
#    fn=>'\\exp (-0.05x^{2})'},
{# 7
    f=>sub{exp (-0.02*($_[0]-0.5)**2)},
    x=>[-8 .. 3],
    istep=>0.05,
    legendpos=>"-1,0.6",
    spy=>"\\coordinate (spypoint) at (axis cs:0,0.98 );\\coordinate (magnifyglass) at (axis cs:-6.3,0.825);",
#    fn=>'\\exp (-0.01 (x-0.5) ^{2})'},
    fn=>'e^{-0.01 (x-0.5) ^{2}}'},
{# 8
    f=>sub{exp (-0.02*$_[0]**2)},
    x=>[-8 .. 3],
    istep=>0.05,
    legendpos=>"-1,0.60",
    spy=>"\\coordinate (spypoint) at (axis cs:0.5,0.995);\\coordinate (magnifyglass) at (axis cs:-6.4,0.825);",
    fn=>'e^{-0.01 x ^{2}}'},
#    fn=>'\\exp (-0.01 x ^{2})'},
{# 9
    f=>sub{sqrt(0.02+ ($_[0]-0.05)**2)},
    x=>[-4..5],
    istep=>0.02,
    legendpos=>"-1.5,5",
    spy=>"\\coordinate (spypoint) at (axis cs:0.2,0.4);\\coordinate (magnifyglass) at (axis cs:3.8,1.1);",
    fn=>'\\sqrt{0.02+(x-0.05)^2}'},
{# 10
    f=>sub{abs(cos(0.31*$_[0]))},
    x=>[2 .. 28],
    istep=>0.05,
    legendpos=>"-10,.83",
    spy=>"\\coordinate (spypoint) at (axis cs:28.5,.15);\\coordinate (magnifyglass) at (axis cs:20,.50);",
    fn=>'|\\cos(0.4x)|'},
{# 11
    f=>sub{sin($_[0]/2)},
    x=>[-4 .. 4],
    istep=>0.05,
    legendpos=>"-4,1",
    spy=>"\\coordinate (spypoint) at (axis cs:2.5,0.900);\\coordinate (magnifyglass) at (axis cs:2.5,-0.400);",
    fn=>'\\sin(x/2)'},
{# 12
    f=>sub{tan($_[0]/3.25)},
    x=>[-5 .. 5],
    istep=>0.05,
    legendpos=>"-10,34.0",
    spy=>"\\coordinate (spypoint) at (axis cs:7.8,2.4);\\coordinate (magnifyglass) at (axis cs:6,-18.0);",
    fn=>'\\tan(x/3.25)'},
{# 3
    f=>sub{($_[0]>0?1:-1)*(abs($_[0]))**.33333333},
    x=>[-4 .. 4],
    istep=>0.02,
    legendpos=>"-4,1.5",
    spy=>"\\coordinate (spypoint) at (axis cs:1,1);\\coordinate (magnifyglass) at (axis cs:2.5,-.5);",
    #fn=>'\\root 3 \of {x}'},
    fn=>'\\sqrt[3]{x}'},
{# 14
    f=>sub{sqrt (25-$_[0]**2)},
    x=>[-5..5],
    istep=>0.02,
    legendpos=>"0.1,2.5",
    spy=>"\\coordinate (spypoint) at (axis cs:-3.4,3.6);\\coordinate (magnifyglass) at (axis cs:-2.3,1.2);",
    fn=>'\\sqrt{16-x^2}'},
{# 15
    f=>sub{$_[0]*$_[0]},
    x=>[-2.95, -2.6, -2.1, -1.8, -1.4, -1.0, -0.75, -0.3, -0.05, 0.2, 0.55, 0.9, 1.25, 1.6, 1.7, 2.1, 2.4, 3.0],
    istep=>0.05,
    legendpos=>"-2,9.8",
    spy=>"\\coordinate (spypoint) at (axis cs:2.1,5);\\coordinate (magnifyglass) at (axis cs:0,2.8);",
    fn=>'x^2'},
{# 16
    x=>[-2.95, -2.6, -2.1, -1.8, -1.4, -1.0, -0.75, -0.3, -0.05, 0.2, 0.55, 0.9, 1.25, 1.6, 1.7, 2.1, 2.4, 3.0],
    f=>sub{$_[0]*$_[0]*$_[0]*$_[0]},
    istep=>0.05,
    legendpos=>"-2.36,85",
    spy=>"\\coordinate (spypoint) at (axis cs:2.4,33);\\coordinate (magnifyglass) at (axis cs:1.35,60);",
    fn=>'x^4'},
{# 17
    x=>[-2.95, -2.6, -2.1, -1.8, -1.4, -1.0, -0.75, -0.3, -0.05, 0.2, 0.55, 0.9, 1.25, 1.6, 1.7, 2.1, 2.4, 3.0],
    f=>sub{exp(-$_[0]*$_[0]/2)},
    istep=>0.05,
    legendpos=>"-3.3,1.1",
    spy=>"\\coordinate (spypoint) at (axis cs:-0.4,.94);\\coordinate (magnifyglass) at (axis cs:0,.25);",
    fn=>'\\exp(x^2/2)'},
{# 18
    x=>[-2.95, -2.6, -2.1, -1.8, -1.4, -1.0, -0.75, -0.3, -0.05, 0.2, 0.55, 0.9, 1.25, 1.6, 1.7, 2.1, 2.4, 3.0],
    f=>sub{tanh(-$_[0])},
    istep=>0.05,
    legendpos=>"-3,0",
    spy=>"\\coordinate (spypoint) at (axis cs:-1,.75);\\coordinate (magnifyglass) at (axis cs:2,.5);",
    fn=>'\\tanh(x)'},
{# 19
    x=>[-2.95, -2.6, -2.1, -1.8, -1.4, -1.0, -0.75, -0.3, -0.05, 0.2, 0.55, 0.9, 1.25, 1.6, 1.7, 2.1, 2.4, 3.0],
    f=>sub{sin(-$_[0])},
    istep=>0.05,
    legendpos=>"-3,0",
    spy=>"\\coordinate (spypoint) at (axis cs:-1.3,.95);\\coordinate (magnifyglass) at (axis cs:1.5,.5);",
    fn=>'\\sin(x)'},
{# 20
    x=>[5 .. 30],
    f=>sub{1/(1.1+sin($_[0]/2.5))},
    istep=>0.05,
    legendpos=>"-3,0",
    spy=>"\\coordinate (spypoint) at (axis cs:-1.3,.95);\\coordinate (magnifyglass) at (axis cs:1.5,.5);",
    fn=>'\\sin^6(0.2x)'},
);

my $imax=0;
foreach my $t (0.. $#task){
    my @x=@{$task[$t]{x}};
    $task[$t]{y}=[map {&{$task[$t]{f}}($_)} @x];
    $imax=$#x if $imax<$#x;}

my @data;
my $src=sprintf<<".", $imax+1,$imax+1;
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_spline.h>

int
main (void)
{
  int i;
  double xi, yi, ii, x[%d], y[%d];

.
my @pperlish = qw(linear ncubic nquad mingled);
my @perlish = (qw(iden exact), @pperlish);
my @gsls=qw"cspline akima steffen";
foreach my $t (0.. $#task){
    $data[$t]={};
    my @x=@{$task[$t]{x}};
    my $f=$task[$t]{f};
    my @y=@{$task[$t]{y}};
    my @yy;
    my $spline = Math::Spline::MingledSquare->new(\@x,\@y);
    $src .= "  printf (\"# [$t]\\n\");\n";
    foreach(@perlish){ $task[$t]{$_}=[]; }
    foreach my $i (0 .. $#x){
            $src .= sprintf ( <<".", $i,  $x[$i], $i, $y[$i], $i, $i);}
      x[%d] = %.9g;
      y[%d] = %.9g;
      printf ("%%f %%f\\n", x[%d], y[%d]);     
.
    foreach(@perlish){ $data[$t]{$_}=[]; }
    my $m1=($y[1]-$y[0])/($x[1]-$x[0]);
    my $m2=($y[2]-$y[1])/($x[2]-$x[1]);
    my $de1=($m2-$m1)/($x[2]-$x[0]);
    my $m0; 
    my $de0;
    my $dem;
    my $mm;
    foreach my $i (0 .. $#x-1){
        my $mmm; my $demm;
        $mmm=$mm; $mm=$m0; $m0=$m1; $m1=$m2; $demm=$dem; $dem=$de0; $de0=$de1; 
        if ($i<$#x-2){
            $m2=($y[$i+3]-$y[$i+2])/($x[$i+3]-$x[$i+2]);
            $de1=($m2-$m1)/($x[$i+3]-$x[$i+1]);    }
        my $dxx=($x[$i+1]-$x[$i])/12;
        my $dy=($y[$i+1]-$y[$i]);
        foreach(@perlish){ $data[$t]{$_}[$i]=[]; }
        foreach my $j (0 .. 12){
            my $xx=$x[$i]+$j*$dxx;
            $data[$t]{iden}[$i][$j]=$xx;
            $data[$t]{exact}[$i][$j]=&$f($xx);
            $data[$t]{linear}[$i][$j]=$y[$i]+$j*$dy/12; 
            $data[$t]{mingled}[$i][$j]=$spline->evaluate($xx);
            $data[$t]{ncubic}[$i][$j]=
                $i<1?       $y[0]+$j*$dxx*($m0 +($j-12)*$dxx*($de0+($x[0]+$j*$dxx-$x[2])*($de1-$de0)/($x[3]-$x[0]))) :
                $i<$#x-1?   $y[$i-1]+($j*$dxx+$x[$i]-$x[$i-1])*($mm +$j*$dxx*($dem+($j-12)*$dxx*($de0-$dem)/($x[$i+2]-$x[$i-1]))) :
                            $y[$#x-1]+($j*$dxx)*($m0 +($j-12)*$dxx*($dem+($j*$dxx+$x[$#x-1]-$x[$#x-2])*($dem-$demm)/($x[$#x]-$x[$#x-3])));
            $data[$t]{nquad}[$i][$j]=
                $i<1?       $y[0]+$j*$dxx*($m0 +($j-12)*$dxx*($de0+($x[0]+6*$dxx-$x[2])*($de1-$de0)/($x[3]-$x[0]))) :
                $i<$#x-1?   $y[$i]+($j*$dxx)*($m0 +($j-12)*$dxx*($de0 +(($x[$i]+$x[$i+1])/2-$x[$i+2])*($de0-$dem)/($x[$i+2]-$x[$i-1]))) :
                            $y[$#x-1]+($j*$dxx)*($m0 +($j-12)*$dxx*($dem+(6*$dxx+$x[$#x-1]-$x[$#x-2])*($dem-$demm)/($x[$#x]-$x[$#x-3])));
  }}
    
    my $xnn= $#x+1;
    foreach my $sname (@gsls){ 
        $src .= sprintf <<".";
  printf ("#$sname\\n");

  {
    gsl_interp_accel *acc
      = gsl_interp_accel_alloc ();
    gsl_spline *spline
      = gsl_spline_alloc (gsl_interp_$sname, $xnn);

    gsl_spline_init (spline, x, y, $xnn);

.
    foreach my $i (0 .. $#x-1){
        my $dxx=($x[$i+1]-$x[$i])/12;
        foreach my $j ( 0 .. 11){
            $src .=sprintf  
            "        printf (\"$i $j %%.9f\\n\", gsl_spline_eval (spline, %.4f, acc));\n", $x[$i]+$j*$dxx;}}
        $src .= <<".";
    gsl_spline_free (spline);
    gsl_interp_accel_free (acc);
  }
.
}}
$src .= "    return 0;\n}\n";
open O, ">tests.c" or die "$! tests.c\n";
print O $src;
close O;
system "gcc -lm  -lgsl -lcblas tests.c;./a.out>gsl.data";
#print Dumper \@data;

my $function;
my %metric;
foreach my $k(qw(C L L2)){
    $metric{$k}=[];
    foreach my $fn(0 .. $#task){
        $metric{$k}[$fn]={};
        foreach (@perlish , @gsls){
            $metric{$k}[$fn]{$_}=0}}}
open I, "gsl.data" or die $!;
my $sname;
my $t;
while (<I>){
    chomp;
    if (/^\#(\S.*)/){
        $sname=$1;
        $data[$t]{$sname} =[] unless $data[$t]{$sname}; 
        next}
    elsif (/^# \[(.+)\]/){
        $sname='';
        $t=$1;
        next}
    next unless $_ and $sname;
    s/^\-0\.00 /0.00 /;
    die "$_\n" unless /([0-9]+) ([0-9]+) ([-0-9\.e]+)/;
    unless ($data[$t]{$sname}[$1]){
        $data[$t]{$sname}[$1]=[] ;
        $data[$t]{$sname}[$1][12]=$task[$t]{y}[$1+1] ;}
    $data[$t]{$sname}[$1][$2]=$3; }
close I;

foreach $t (0 .. $#task){
     my @x=@{$task[$t]{x}};
    my @y=@{$task[$t]{y}};
    open W, ">data-$t.data";
    print W join( "\t", @perlish, @gsls)."\n";
    foreach my $i (0 .. $#x-1){
        my $a='';
        my $dx=($x[$i+1]-$x[$i]);
        my $dy=($y[$i+1]-$y[$i]);
        foreach my $j (0 .. 11){
            my $x=$x[$i]+$j*$dx/12;
            print W join( "\t", map{$task[$t]{$_}[$i][$j] || $data[$t]{$_}[$i][$j] } @perlish, @gsls)."\n";
            foreach my $sname (keys %{$data[$t]}){
                next if $sname eq 'iden';
                my $er=abs($data[$t]{exact}[$i][$j]-$data[$t]{$sname}[$i][$j]);
                #  print "metric{C}[$t]{$sname}=$er if $metric{C}[$t]{$sname}<$er\n";
                $metric{C}[$t]{$sname}=$er if $metric{C}[$t]{$sname}<$er;
                $metric{L}[$t]{$sname}+=$er*$dx/12; 
                $metric{L2}[$t]{$sname}+=$er*$er*$dx/12;}}}
         print W join( "\t",$task[$t]{x}[$#x], map{$task[$t]{y}[$#x] } 0 .. $#perlish+ $#gsls)."\n";
    close W}
my @decor= qw(exact best second third forth medium last);
my $table= #join "\n",(map {"\\definecolor{$_}{rgb}{0,0.1,1}\n\\def\\$_\#1{\\textcolor{$_}{\#1}}"} @decor),
    "\\begin{tabular}{rcrrrrrrr}\n\\toprule\n mesh&formula&"
    . join ('&', map {"\u$_"} @gsls, @pperlish)."\\\\\n\\midrule";
my $deparse=B::Deparse->new();
foreach my $t (0 .. $#task){
    my $l2=$metric{L2}[$t];
    my $txt=$deparse->coderef2text($task[$t]{f});
    $txt=~s/use.*//mg;
    $txt=~s/\n//sg;
    $txt=~s/    //sg;
    my @scor=sort {$l2->{$a}<=>$l2->{$b}} keys %$l2;
    my $best_local=$l2->{akima}>$l2->{steffen}? $l2->{steffen} : $l2->{akima};
    $best_local=$l2->{ncubic} if $l2->{ncubic}< $best_local;
    my %place;
    foreach (0 .. $#scor){$place{$scor[$_]}=$_};
    $table.=(join '&\\', "\\hyperref[fig:pic$t]{".($task[$t]{x}[0]==-2.95 ? "CD" : "$task[$t]{x}[0]..$task[$t]{x}[-1]")."} &\$ $task[$t]{fn}\$",
        (map {$decor[(
            $l2->{$_} < 0.5 * $best_local ? 0 : 
            abs(($best_local/$l2->{$_})-($l2->{$_}/$best_local)) < 2-1/2 ? 1 :
            abs(($best_local/$l2->{$_})-($l2->{$_}/$best_local)) < 5-1/5 ? 2 :
            abs(($best_local/$l2->{$_})-($l2->{$_}/$best_local)) < 10-1/10 ? 3 :
            abs(($best_local/$l2->{$_})-($l2->{$_}/$best_local)) < 12-1/20 ? 4 :
            abs(($best_local/$l2->{$_})-($l2->{$_}/$best_local)) < 100 ? 5 :
            6
            )]."\{". (abs($l2->{$_})<0.0000000001? '0}' :sprintf "%.2g}",$l2->{$_})} @gsls,@pperlish)). "\\\\[1mm]\n";
#     printf "%.2f%% (%s),\t%.2f%% (%s),\t%.2f%% (%s),\t%.2f%% (%s) $txt\n",100* $l2->{$scor[1]}/$l2->{$scor[2]},$scor[1],100* $l2->{$scor[3]}/$l2->{$scor[2]},$scor[3],
#      100* $l2->{$scor[4]}/$l2->{$scor[2]},$scor[4],100* $l2->{$scor[5]}/$l2->{$scor[2]},$scor[5];
#                   $a.=sprintf "%.9f ", $data[$t]{$sname}[$i][$j]-$data[$t]{linear}[$i][$j];}
#           printf W "$a %.9f\n", $data[$t]{y}[$i];   print join "\t", (map {$_ eq 'exact'? '' : sprintf "%.6f (%s)", $l2->{$_}, $_} sort {$l2->{$a}<=>$l2->{$b}} keys %$l2), "$txt\n";

    my $path=join ' ', 'coordinates {', map ({"($task[$t]{x}[$_],$task[$t]{y}[$_])"} 0 .. $#{$task[$t]{x}}), '}';
    my $tex=<<'.'
\batchmode
\documentclass[10pt,a4paper,landscape]{article}
\usepackage[T2A]{fontenc}
\usepackage[utf8]{inputenc}
\usepackage[russian]{babel}
\usepackage{amssymb}
\usepackage{xcolor}
\usepackage{array}
\usepackage{multirow}
\usepackage{booktabs}
\textwidth 10cm
\textheight 8cm
\topmargin -2.5cm
\def\A{{\tiny A}}
\def\S{{\tiny S}}
\def\C{{\tiny C}}
\def\M{{\tiny M}}
\def\N{{\tiny N}}
\def\Q{{\tiny Q}}
\oddsidemargin=-1cm \evensidemargin=-1cm 
\definecolor{Akima}{rgb}{0,0.1,1}
\definecolor{Cspline}{rgb}{0.5,0.6,0}
\definecolor{Linear}{rgb}{0.5,0.5,0.5}
\definecolor{Mingled}{rgb}{.9,0,0}
\definecolor{Steffen}{rgb}{0,0.6,0.5}
\definecolor{Ncubic}{rgb}{0,0,0.6}
\definecolor{Nquad}{rgb}{0.6,0,0}
\thispagestyle{empty}
\usepackage{pgfplots}
\usetikzlibrary{pgfplots.groupplots}
\pgfplotsset{compat=newest}
\usetikzlibrary{plotmarks}
\usetikzlibrary{spy}
\begin{document}
\vbox{
\begin{tikzpicture}%[spy using outlines={circle, magnification=4, connect spies}]        
\begin{groupplot}[group style={group name=my plots,group size=1 by 2,xticklabels at=edge bottom,ylabels at=edge left,yticklabels at=edge left,vertical sep=11mm},footnotesize,width=100mm,height=50mm,
%,xmin=-0.5,xmax=10.5,ymin=0,ymax=30,ytick={0,10,...,20},
minor xtick={
.
    . join ("," , @{$task[$t]{x}}). <<".";
},  tickpos=left,ytick align=outside,xtick align=outside, grid=minor]
\\nextgroupplot[legend to name={CommonLegend},legend style={legend columns=4}]
\\addplot[color=black, thick]  table [x=iden, y=exact]{./data-$t.data};%
\\addlegendentry{Exact};%
\\addplot[color=Linear, thin, sharp plot, color=Linear] table [x=iden, y=linear] {./data-$t.data};% 
\\addlegendentry{Linear};%
\\addplot[color=Cspline, thin, mark=text, text mark=\\C, mark size=1, mark repeat=12, mark phase=6, mark color=Cspline] table [x=iden, y=cspline]{./data-$t.data};%
\\addlegendentry[]{Cspline};%
%\\addplot[color=Steffen, thin, mark=text, text mark=\\S, mark size=.9, mark repeat=12, mark phase=4, mark color=Steffen] table [x=iden, y=steffen]{./data-$t.data};%
%\\addlegendentry{Steffen};%
\\addplot[color=Akima, thin, mark=text, text mark=\\A, mark size=.8, mark repeat=12, mark phase=8, mark color=Akima] table [x=iden, y=akima]{./data-$t.data};%
\\addlegendentry{Akima};%
\\addplot[color=Ncubic, thin, mark=text, text mark=\\N, mark size=0.7, mark repeat=12, mark phase=11, mark color=Ncubic] table [x=iden, y=ncubic]{./data-$t.data};%
\\addlegendentry{Ncubic};%
\\addplot[color=Nquad, thin, mark=text, text mark=\\Q, mark size=0.7, mark repeat=12, mark phase=10, mark color=Nquad] table [x=iden, y=ncubic]{./data-$t.data};%
\\addlegendentry{Nquad};%
\\addplot[color=Mingled, thin, mark=text, text mark=\\M, mark size=0.7, mark repeat=12, mark phase=2, mark color=Mingled] table [x=iden, y=mingled]{./data-$t.data};%
\\addlegendentry{Mingled};%
\\nextgroupplot
.
#    print Dumper \@data;
    foreach my $i (0 .. $#{$task[$t]{x}}-1){
    $tex .=sprintf <<'.',  
\addplot[thin, color=black, thick,  ] coordinates {%s};%%
\addplot[thin, color=Cspline,  mark=text, mark color=Cspline, text mark=\C, mark repeat=12, mark phase=6] coordinates {%s};%%
\addplot[thin, color=Akima,    mark=text, mark color=Akima,   text mark=\A, mark size=.8, mark repeat=12, mark phase=8] coordinates {%s};%%
\addplot[thin, color=Ncubic,  mark=text, mark color=Ncubic, text mark=\N, mark size=.7, mark repeat=12, mark phase=10] coordinates {%s};%%
\addplot[thin, color=Nquad,  mark=text, mark color=Nquad, text mark=\Q, mark size=.7, mark repeat=12, mark phase=11] coordinates {%s};%%
\addplot[thin, color=Mingled,  mark=text, mark color=Mingled, text mark=\M, everymark {scale=.7}, mark repeat=12, mark phase=2] coordinates {%s};%%
.
    "($data[$t]{iden}[0][0],0) ($data[$t]{iden}[-1][-1],0)",
    join (' ', map {sprintf "(%.7f,%.7f)", $data[$t]{iden}[$i][$_], $data[$t]{cspline}[$i][$_] - $data[$t]{exact}[$i][$_]} 0 .. 12),
    join (' ', map {sprintf "(%.7f,%.7f)", $data[$t]{iden}[$i][$_], $data[$t]{akima}[$i][$_]   - $data[$t]{exact}[$i][$_]} 0 .. 12),    
    join (' ', map {sprintf "(%.7f,%.7f)", $data[$t]{iden}[$i][$_], $data[$t]{ncubic}[$i][$_]  - $data[$t]{exact}[$i][$_]} 0 .. 12),
    join (' ', map {sprintf "(%.7f,%.7f)", $data[$t]{iden}[$i][$_], $data[$t]{nquad}[$i][$_]  - $data[$t]{exact}[$i][$_]} 0 .. 12),
    join (' ', map {sprintf "(%.7f,%.7f)", $data[$t]{iden}[$i][$_], $data[$t]{mingled}[$i][$_] - $data[$t]{exact}[$i][$_]} 0 .. 12);}
    $tex .=<<'.';
\end{groupplot}
\end{tikzpicture}
\vskip -50mm\rlap{\hskip 24mm\ref{CommonLegend}}\vss}
\end{document}
.

if(1){
    open OUT1, ">pic_$t.tex" or die $!;
    print  OUT1 $tex;
    close OUT1;
    `pdflatex pic_$t`; 
    die unless -f "pic_$t.pdf"; 
    `pdfcrop pic_$t.pdf;mv pic_$t-crop.pdf pic_$t.pdf; `;}
}
open T, ">table.tex";
print T $table."\\bottomrule\n\\end{tabular}\n";
close T;

__END__

